@@ -3,6 +3,8 @@ from distutils.version import LooseVersion
 from redmine.utilities import to_string
 from redmine.managers import ResourceManager
 from redmine.exceptions import (
+    ValidationError,
+    ForbiddenError,
     ResourceAttrError,
     ReadonlyAttrError,
     CustomFieldValueError,
@@ -213,8 +215,8 @@ class _Resource(object):
             self._attributes['custom_fields'].extend(value)
             self._changes[item] = self._attributes['custom_fields']
         else:
-            value = self.manager.prepare_params({item: value})[item]
-            self._changes[item] = value
+            prep_item, prep_value = self.manager.prepare_params({item: value}).popitem()
+            self._changes[prep_item] = prep_value
             self._attributes[item] = value
 
             if item in _RESOURCE_SINGLE_ATTR_ID_MAP:
@@ -332,7 +334,7 @@ class Project(_Resource):
     query_update = '/projects/{0}.json'
     query_delete = '/projects/{0}.json'
 
-    _includes = ('trackers', 'issue_categories')
+    _includes = ('trackers', 'issue_categories', 'enabled_modules')
     _relations = (
         'wiki_pages',
         'memberships',
@@ -352,7 +354,12 @@ class Project(_Resource):
         if item == 'parent' and item in self._attributes:
             return ResourceManager(self.manager.redmine, 'Project').to_resource(self._attributes[item])
 
-        return super(Project, self).__getattr__(item)
+        value = super(Project, self).__getattr__(item)
+
+        if item == 'enabled_modules':
+            value = [module.get('name') if isinstance(module, dict) else module for module in value]
+
+        return value
 
 
 class Issue(_Resource):
@@ -835,6 +842,12 @@ class CustomField(_Resource):
         if item == 'value' and item not in self._attributes:
             return 0
 
+        # Redmine <2.5.2 returns only single tracker instead of a list of
+        # all available trackers, see http://www.redmine.org/issues/16739
+        # for details
+        elif item == 'trackers' and 'tracker' in self._attributes[item]:
+            self._attributes[item] = [self._attributes[item]['tracker']]
+
         return super(CustomField, self).__getattr__(item)
 
 
@@ -895,7 +908,10 @@ class Contact(_Resource):
             try:
                 return self._redmine.request('post', url, data={'project': {'id': project_id}})
             except ResourceNotFoundError:
-                raise ResourceRequirementsError((('CRM plugin', '3.4.0'),))
+                raise ValidationError("Attempt to add contact to a project that doesn't exist")
+            except ForbiddenError:
+                raise ValidationError(
+                    'Attempt to add contact to a project that either has contacts module disabled or is read-only')
 
         def remove(self, project_id):
             """Removes project from contact's project list"""
@@ -904,11 +920,14 @@ class Contact(_Resource):
             try:
                 return self._redmine.request('delete', url)
             except ResourceNotFoundError:
-                raise ResourceRequirementsError((('CRM plugin', '3.4.0'),))
+                raise ValidationError("Attempt to remove contact from a project that doesn't exist")
+            except ForbiddenError:
+                raise ValidationError(
+                    'Attempt to remove contact from a project that either has contacts module disabled or is read-only')
 
     @classmethod
     def translate_params(cls, params):
-        if 'tag_list' in params:
+        if isinstance(params.get('tag_list'), (list, tuple)):
             params['tag_list'] = ','.join(params['tag_list'])
 
         if 'phones' in params:
@@ -932,13 +951,6 @@ class Contact(_Resource):
 
         return super(Contact, self).__getattr__(item)
 
-    def __setattr__(self, item, value):
-        if item in ('phones', 'emails', 'tag_list'):
-            self._attributes[item] = value
-            self._changes.update(self.translate_params({item: value}))
-        else:
-            super(Contact, self).__setattr__(item, value)
-
     def __str__(self):
         try:
             return super(Contact, self).__str__()
